@@ -8,18 +8,8 @@
 //!
 //! # Interoptopus 🐙
 //!
-//! The polyglot bindings generator for your library (C#, C, Python, ...)
+//! The polyglot bindings generator for your library.
 //!
-//! <br>
-//!
-//! Huh?
-//! - You wrote this cool Rust library and want everyone else to use it.
-//! - But everyone else runs Unity, C, Python, ...
-//! - "Not a problem", you say, "I'll just use Interoptopus".
-//!
-//! And you'll live happily<sup>*</sup> ever after.
-//!
-//! <sub>*Actual results may depend on other life choices.</sub>
 //!
 //! ## Code you write ...
 //!
@@ -47,61 +37,67 @@
 //!
 //! | Language | Crate | Sample Output |
 //! | --- | --- | --- |
-//! | C# (incl. Unity) | [**interoptopus_backend_csharp**](https://crates.io/crates/interoptopus_backend_csharp) | [Interop.cs](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_csharp/tests/output/Interop.cs) |
-//! | C | [**interoptopus_backend_c**](https://crates.io/crates/interoptopus_backend_c) | [my_header.h](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_c/tests/output/my_header.h) |
-//! | Python [CFFI](https://cffi.readthedocs.io/en/latest/index.html) | [**interoptopus_backend_cpython_cffi**](https://crates.io/crates/interoptopus_backend_cpython_cffi) | [reference.py](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_cpython_cffi/tests/output/reference_project.py) |
-//! | Your language | Write your own backend<sup>1</sup> | - |
+//! | C# | [**interoptopus_backend_csharp**](https://crates.io/crates/interoptopus_backend_csharp) | [Interop.cs](https://github.com/ralfbiedert/interoptopus/blob/master/backends/csharp/tests/output/Interop.cs) |
+//! | C | [**interoptopus_backend_c**](https://crates.io/crates/interoptopus_backend_c) | [my_header.h](https://github.com/ralfbiedert/interoptopus/blob/master/backends/c/tests/output/my_header.h) |
+//! | Python<sup>1</sup> | [**interoptopus_backend_cpython_cffi**](https://crates.io/crates/interoptopus_backend_cpython_cffi) | [reference.py](https://github.com/ralfbiedert/interoptopus/blob/master/backends/cpython_cffi/tests/output/reference_project.py) |
+//! | Other | Write your own backend<sup>2</sup> | - |
 //!
-//! <sup>1</sup> Create your own backend in just a few hours. No pull request needed. [Pinkie promise](https://github.com/ralfbiedert/interoptopus/blob/master/FAQ.md#new-backends).
+//! <sup>1</sup> Using Python [CFFI](https://cffi.readthedocs.io/en/latest/index.html). <br>
+//! <sup>2</sup> Create your own backend in just a few hours. No pull request needed. [Pinkie promise](https://github.com/ralfbiedert/interoptopus/blob/master/FAQ.md#new-backends).
 //!
 //! ## Getting Started 🍼
 //!
-//! If you ...
-//! - want to **create a new API** see the [**example projects**](https://github.com/ralfbiedert/interoptopus/tree/master/examples),
-//! - need to **support a new language** or rewrite a backend, [**copy and adapt the C backend**](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_backend_c).
+//! If you want to ...
+//! - **create a new API** see the [**hello world**](https://github.com/ralfbiedert/interoptopus/tree/master/examples/hello_world),
+//! - **understand what's possible**, see the [**reference project**](https://github.com/ralfbiedert/interoptopus/tree/master/reference_project/src),
+//! - **support a new language**, [**copy the C backend**](https://github.com/ralfbiedert/interoptopus/tree/master/backends/c).
 //!
 //! ## Features
 //!
-//! - explicit, type-safe, **single source of truth** API definition in Rust,
-//! - **quality-of-life [patterns](crate::patterns)** on **both sides** (e.g., [options](crate::patterns::option), [slices](crate::patterns::slice), [services](crate::patterns::service), ...)
-//! - **minimal on dependencies**, build time, tooling impact,
-//! - if your **project compiles your bindings should work**<sup>&#42;*cough*&#42;</sup> (i.e., generated and callable),
-//! - **extensible**, multiple backends, **easy to support new languages**, fully **customizable**,
-//! - **no scripts needed**, `cargo build` + `cargo test` **can produce and test** (if lang installed) generated bindings
+//! - explicit, **single source of truth** API definition in Rust,
+//! - **quality-of-life [patterns](crate::patterns)** (e.g., [slices](crate::patterns::slice), [services](crate::patterns::service), ...)
+//! - if your **project compiles your bindings should work**, <sup>&#42;*cough*&#42;</sup>
+//! - easy to support new languages, fully **customizable**,
+//! - **no scripts needed**, just `cargo build` + `cargo test`.
 //!
 //!
 //! Gated behind **feature flags**, these enable:
 //!
-//! - `derive` - Proc macros such as `ffi_constant`, `ffi_function`, `ffi_type`.
-//! - `testing` - Functions to test generated Python, C#, C from Unit tests.
+//! - `derive` - Proc macros such as `ffi_type`, ...
 //! - `serde` - Serde attributes on internal types.
-//! - `log` - Invoke `log` on FFI errors (you still need actual logger).
+//! - `log` - Invoke [log](https://crates.io/crates/log) on FFI errors.
 //!
 //!
 //! ## Supported Rust Constructs
-//! See the [**reference project**](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src); it lists all supported constructs including:
-//! - [functions](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/functions.rs) (`extern "C"` functions and delegates)
-//! - [types](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/types.rs) (primitives, composite, enums (numeric only), opaques, references, pointers, ...)
-//! - [constants](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/constants.rs) (primitive constants; results of const evaluation)
-//! - [patterns](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src/patterns) (ASCII pointers, options, slices, classes, ...)
 //!
-//! As a rule of thumb we recommend to be slightly conservative with your signatures and always "think C", since other languages don't track lifetimes
-//! well and it's is easy to accidentally pass an outlived pointer or doubly alias a `&mut X` on reentrant functions.
+//! See the [**reference project**](https://github.com/ralfbiedert/interoptopus/tree/master/reference_project/src) for an overview:
+//! - [functions](https://github.com/ralfbiedert/interoptopus/blob/master/reference_project/src/functions.rs) (`extern "C"` functions and delegates)
+//! - [types](https://github.com/ralfbiedert/interoptopus/blob/master/reference_project/src/types.rs) (composites, enums, opaques, references, ...)
+//! - [constants](https://github.com/ralfbiedert/interoptopus/blob/master/reference_project/src/constants.rs) (primitive constants; results of const evaluation)
+//! - [patterns](https://github.com/ralfbiedert/interoptopus/tree/master/reference_project/src/patterns) (ASCII pointers, options, slices, classes, ...)
+//!
 //!
 //!
 //! ## Performance 🏁
 //!
-//! Generated low-level bindings should be "zero cost" w.r.t. hand-crafted bindings for that language. However, even hand-crafted bindings
-//! have an inherent, language-specific cost. For C# that cost can be almost 0, for Python CFFI it can be high. Patterns and convenience
-//! helpers might add additional overhead.
+//! Generated low-level bindings are "zero cost" w.r.t. hand-crafted bindings for that language.
 //!
-//! If you **need API design guidance** the following (wip) [**C# call-cost table**](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_csharp/benches/BENCHMARK_RESULTS.md)<sup>🔥</sup> can help.
+//! That said, even hand-crafted bindings encounter some target-specific overhead
+//! at the FFI boundary (e.g., marshalling or pinning in managed languages) For C# that cost
+//! can be nanoseconds, for Python CFFI it can be microseconds.
+//!
+//! See this [**C# call-cost table**](https://github.com/ralfbiedert/interoptopus/blob/master/backends/csharp/benches/BENCHMARK_RESULTS.md)<sup>🔥</sup> for ballpark figures.
 //!
 //! ## Changelog
 //!
+//! - **v0.8** - Moved testing functions to respective backends.
+//! - **v0.7** - Make patterns proc macros for better FFI docs.
+//! - **v0.6** - Renamed and clarified many patterns.
+//! - **v0.5** - More ergonomic slice usage in Rust and FFI.
+//! - **v0.4** - Enable logging support in auto-generated FFI calls.
 //! - **v0.3** - Better compatibility with generics.
-//! - **v0.2** - Introduced "patterns"; produces generally _working_ interop for C#.
-//! - **v0.1** - Has generated C#, C, Python-CFFI bindings at least once.
+//! - **v0.2** - Introduced "patterns"; _working_ interop for C#.
+//! - **v0.1** - First version.
 //!
 //!
 //! ## FAQ
@@ -112,8 +108,7 @@
 //!
 //! PRs are welcome.
 //!
-//! - Bug fixes can be submitted directly. Major changes should be filed as issues
-//! first.
+//! - Submit small bug fixes directly. Major changes should be issues first.
 //! - Anything that makes previously working bindings change behavior or stop compiling
 //! is a major change;
 //! - This doesn't mean we're opposed to breaking stuff just that
@@ -140,8 +135,6 @@ mod core;
 mod error;
 mod generators;
 pub mod patterns;
-#[cfg(feature = "testing")]
-#[cfg_attr(docsrs, doc(cfg(feature = "testing")))] // does this work?
 pub mod testing;
 pub mod util;
 pub mod writer;
